/*******************************************************************
*
*  DESCRIPTION: Atomic Model PairedDeviceFinder
*
*  AUTHOR: Shafagh Jafer
*
*  EMAIL: sjafer@connect.carleton.ca
*
*
*  DATE: 8/06/2006
*
*******************************************************************/

/** include files **/
#include <math.h>            // fabs( ... )
#include <stdlib.h>
#include "PairedDeviceFinder.h"  // base header
#include "message.h"       // InternalMessage ....
#include "mainsimu.h"      // class MainSimulator
#include "strutil.h"       // str2float( ... )

/*******************************************************************
* Function Name: PairedDeviceFinder
* Description: constructor
********************************************************************/
PairedDeviceFinder::PairedDeviceFinder( const string &name )
: Atomic( name )
, search( addInputPort( "search" ) )
, IDin( addInputPort( "IDin" ) )
, getID( addOutputPort( "getID" ) )
, deviceID( addOutputPort( "deviceID" ) )
, processingTime (0,0,2,0)
{
	
	string time1( MainSimulator::Instance().getParameter( description(), "processingTime" ) ) ;

	if( time1 != "" )
		processingTime = time1 ;

}

/*******************************************************************
* Function Name: externalFunction
* Description: the PairedDeviceFinder receives search, IDin
********************************************************************/
Model &PairedDeviceFinder::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
  
    if (msg.port() == search)
     {
        reqSearch = 1;  //set it to True
        holdIn(active, processingTime);
     }
    if (msg.port() == IDin )
     {
       reqSearch = 0;  //set it to False
       receiverID = static_cast < int > (msg.value()); //get the ReceiverID from the receiver
       holdIn(active, processingTime);
     }
     
	
  }
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &PairedDeviceFinder::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &PairedDeviceFinder::outputFunction( const InternalMessage &msg )
{

      if ( reqSearch == 1 )
        {
         sendOutput( msg.time(),getID, reqSearch);
        }
         
        else
        {
         sendOutput( msg.time(),deviceID,receiverID);
        }
      
	return *this;
}


