/*******************************************************************
*
*  DESCRIPTION: Atomic Model ReceiverMobile
*
*  AUTHOR: Shafagh Jafer
*
*  EMAIL: sjafer@connect.carleton.ca
*
*
*  DATE: 8/06/2006
*
*******************************************************************/

/** include files **/
#include <math.h>            // fabs( ... )
#include <stdlib.h>
#include "ReceiverMobile.h"  // base header
#include "message.h"       // InternalMessage ....
#include "mainsimu.h"      // class MainSimulator
#include "strutil.h"       // str2float( ... )

/*******************************************************************
* Function Name: ReceiverMobile
* Description: constructor
********************************************************************/
ReceiverMobile::ReceiverMobile( const string &name )
: Atomic( name )
, IDrequested( addInputPort( "IDrequested" ) )
, permit( addInputPort( "permit" ) )
, data( addInputPort( "data" ) )
, IDout( addOutputPort( "IDout" ) )
, accepted( addOutputPort( "accepted" ) )
, gotData( addOutputPort( "gotData" ) )
, newMsgArrival( addOutputPort( "newMsgArrival" ) )
, permissionTime (0,0,2,0)
, transferTime (0,0,5,0)
{
	string time1( MainSimulator::Instance().getParameter( description(), "permissionTime" ) ) ;

	if( time1 != "" )
		permissionTime = time1 ;
	
	string time2( MainSimulator::Instance().getParameter( description(), "transferTime" ) ) ;

	if( time2 != "" )
		transferTime = time2 ;
	

}

/*******************************************************************
* Function Name: externalFunction
* Description: the ReceiverMobile receives permissionReq, accessReq, or data
********************************************************************/
Model &ReceiverMobile::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
  
    if (msg.port() == IDrequested)
     {
      ID_requested = 1;            //set it to True
      permission_given = 0;     //set it to False
      gotMsg = 0;                     //set it to False
      holdIn(active, permissionTime);
     }
    if (msg.port() == permit )
     {
       permission_given = 1;   //set it to True
  	   ID_requested = 0;         //set it to False
       gotMsg = 0;                 //set it to False
       holdIn(active, permissionTime);
     }
     if (msg.port() == data )
     {
         gotMsg = 1;                         //set it to True 
         ID_requested = 0;               //set it to False
         permission_given = 0;        //set it to False
         totalMsgs = totalMsgs +1; //increment # of msgs received
		 holdIn(active, transferTime);
       
     }
	
  }
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &ReceiverMobile::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &ReceiverMobile::outputFunction( const InternalMessage &msg )
{

      if ( ID_requested == 1 )
        {
         sendOutput( msg.time(),IDout, ID);
        }
         
        if ( permission_given == 1 )
        {
         sendOutput( msg.time(),accepted,permission_given);
        }
      if (gotMsg == 1)
        {
         sendOutput (msg.time(),gotData,1);
         sendOutput (msg.time(),newMsgArrival,totalMsgs);
         }
	return *this;
}


