/*******************************************************************
*
*  DESCRIPTION: Atomic Model SenderMobile
*
*  AUTHOR: Shafagh Jafer
*
*  EMAIL: sjafer@connect.carleton.ca
*
*
*  DATE: 8/06/2006
*
*******************************************************************/

/** include files **/
#include <math.h>            // fabs( ... )
#include <stdlib.h>
#include "SenderMobile.h"  // base header
#include "message.h"       // InternalMessage ....
#include "mainsimu.h"      // class MainSimulator
#include "strutil.h"       // str2float( ... )

/*******************************************************************
* Function Name: SenderMobile
* Description: constructor
********************************************************************/
SenderMobile::SenderMobile( const string &name )
: Atomic( name )
, enableBT( addInputPort( "enableBT" ) )
, deviceFound( addInputPort( "deviceFound" ) )
, start( addInputPort( "start" ) )
, dataSent( addInputPort( "dataSent" ) )
, searchDevice( addOutputPort( "searchDevice" ) )
, connect( addOutputPort( "connect" ) )
, dataOut( addOutputPort( "dataOut" ) )
, permissionTime (0,0,2,0)
, transferTime (0,0,5,0) 
, processingTime (0,0,2,0)
{
	string time1( MainSimulator::Instance().getParameter( description(), "permissionTime" ) ) ;

	if( time1 != "" )
		permissionTime = time1 ;
	
	string time2( MainSimulator::Instance().getParameter( description(), "transferTime" ) ) ;

	if( time2 != "" )
		transferTime = time2 ; 
		
	string time3( MainSimulator::Instance().getParameter( description(), "processingTime" ) ) ;

	if( time3 != "" )
		processingTime = time3 ;

}

/*******************************************************************
* Function Name: externalFunction
* Description: the SenderMobile receives permissionOK, accessOK, or dataSent
********************************************************************/
Model &SenderMobile::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
  
    if (msg.port() == enableBT)
     {
        enable = 1;     //set it to True
        sending = 0;   //set it to False
        gotID = 0;      //set it to False

      holdIn(active, permissionTime);
     }
    if (msg.port() == deviceFound )
     {
       gotID = 1;       //set it to True
       sending = 0;   //set it to False
       enable = 0;    //set it to False
       receiverID = static_cast < int > (msg.value()); // got the ID of the receiver
       holdIn(active, processingTime);
     }
     if (msg.port() == start )
     {
       sending = 1;  //set it to True
       gotID = 0;      //set it to False
       enable = 0;    //set it to False
       
       holdIn(active, transferTime);
       
     }
	if (msg.port() == dataSent )
     {
       sending = 0;   //set it back to False
       gotID = 0;      //set it to False
       enable = 0;    //set it to False
       
       
     }
	
  }
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &SenderMobile::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &SenderMobile::outputFunction( const InternalMessage &msg )
{

      if ( enable == 1 )
        {
         sendOutput( msg.time(),searchDevice, 1);
        }
         
        if ( gotID == 1 )
        {
         sendOutput( msg.time(),connect,receiverID);
        }
      if (sending == 1)
        {
         sendOutput (msg.time(),dataOut,data);
        }
	return *this;
}


